define("epi-cms/project/ProjectService", [
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",
    "dojo/promise/all",
    "dojo/Evented",

// epi
    "epi/dependency",
    "epi/shell/xhr/ErrorHandler",
    "epi-cms/core/ContentReference"
],

function (
// dojo
    declare,
    lang,
    when,
    all,
    Evented,

// epi
    dependency,
    ErrorHandler,
    ContentReference
) {

    return declare([Evented], {
        // summary:
        //      A service for interacting with projects
        // tags:
        //      internal

        // projectItemStore: [readonly] Store
        //      A REST store for interacting with project items.
        projectItemStore: null,

        // projectStore: [readonly] Store
        //      A REST store for interacting with projects.
        projectStore: null,

        constructor: function(params) {
            declare.safeMixin(this, params);

            this.projectItemStore = this.projectItemStore || dependency.resolve("epi.storeregistry").get("epi.cms.project.item");
            this.projectStore = this.projectStore || dependency.resolve("epi.storeregistry").get("epi.cms.project");
        },

        getProjectItemsForContent: function(contentLink) {
              // summary:
            //      Get the associated projects items for the given content link
            //
            // contentLink: String
            //      The content link to get the project items for for
            // tags:
            //      internal

             return this.projectItemStore.query({contentLink: contentLink});
        },

        getProjectsForContent: function(contentLink) {
            // summary:
            //      Get the associated projects for the given content link
            //
            // contentLink: String
            //      The content link to get the projects for
            // tags:
            //      internal

            var self = this;

            return when(this.getProjectItemsForContent(contentLink), function(projectItems) {
                // If no project items where found return an empty array
                if(projectItems.length === 0) {
                    return [];
                }

                return self.getProjects(projectItems);
            });
        },

        getProjects: function(projectItems) {
             // summary:
            //      Get the projects for the project items
            //
            // projectItems: int[]
            //      The project items to get the projects for
            // tags:
            //      internal

            var projectStore = this.projectStore;
            var projectPromises = projectItems.map(function(projectItem) {
                return projectStore.get(projectItem.projectId);
            });

            return all(projectPromises);
        },

        addProjectItems: function (projectId, contentReferences) {
            // summary:
            //      Adds the given content references as items of the project.
            //
            // projectId: Number
            //      The target project for adding items to.
            // contentReferences: ContentReference[]
            //      An array of references to content items to add to the project.
            // tags:
            //      public

            // Ensure the content reference we received are valid.
            contentReferences = contentReferences.filter(function (reference) {
                if (ContentReference.isContentReference(reference)) {
                    return true;
                }
                console.warn("epi-cms/project/ProjectService: the given value '" + reference + "' is not a valid content reference.");
            });

            var emit = lang.hitch(this, "emit"),
                store = this.projectItemStore,
                promises = contentReferences.map(function (reference) {
                    return store.add({ contentLink: reference, projectId: projectId });
                }),
                result = all(promises);

            result.then(function(items) {
                    emit("items-added", {
                        items: items
                    });
                })
                .otherwise(ErrorHandler.forXhr);

            return result;
        },

        removeProjectItems: function (projectId, projectItems) {
            // summary:
            //      Removes a project item from a project.
            //
            // projectId: Number
            //      The target project for adding items to.
            // projectItems: ProjectItem[]
            //      An array of project item entities to remove from the
            // tags:
            //      public

            var emit = lang.hitch(this, "emit"),
                remove = lang.hitch(this.projectItemStore, "remove"),
                promises = projectItems.map(function(item) { return remove(item.id); }),
                result = all(promises);

            result.then(function(items) {
                emit("items-removed", {
                    items: projectItems
                });
            }).otherwise(ErrorHandler.forXhr);

            return result;
        }

});
});
